@@ -94,7 +94,9 @@ def return_model(prices, method="mean_historical_return", **kwargs):
         raise NotImplementedError("Return model {} not implemented".format(method))
 
 
-def mean_historical_return(prices, returns_data=False, compounding=True, frequency=252):
+def mean_historical_return(
+    prices, returns_data=False, compounding=True, frequency=252, log_returns=False
+):
     """
     Calculate annualised mean (daily) historical return from input (daily) asset prices.
     Use ``compounding`` to toggle between the default geometric mean (CAGR) and the
@@ -112,6 +114,8 @@ def mean_historical_return(prices, returns_data=False, compounding=True, frequen
     :param frequency: number of time periods in a year, defaults to 252 (the number
                       of trading days in a year)
     :type frequency: int, optional
+    :param log_returns: whether to compute using log returns
+    :type log_returns: bool, defaults to False
     :return: annualised mean (daily) return for each asset
     :rtype: pd.Series
     """
@@ -121,7 +125,7 @@ def mean_historical_return(prices, returns_data=False, compounding=True, frequen
     if returns_data:
         returns = prices
     else:
-        returns = returns_from_prices(prices)
+        returns = returns_from_prices(prices, log_returns)
     if compounding:
         return (1 + returns).prod() ** (frequency / returns.count()) - 1
     else:
@@ -129,7 +133,12 @@ def mean_historical_return(prices, returns_data=False, compounding=True, frequen
 
 
 def ema_historical_return(
-    prices, returns_data=False, compounding=True, span=500, frequency=252
+    prices,
+    returns_data=False,
+    compounding=True,
+    span=500,
+    frequency=252,
+    log_returns=False,
 ):
     """
     Calculate the exponentially-weighted mean of (daily) historical returns, giving
@@ -149,16 +158,19 @@ def ema_historical_return(
     :type frequency: int, optional
     :param span: the time-span for the EMA, defaults to 500-day EMA.
     :type span: int, optional
+    :param log_returns: whether to compute using log returns
+    :type log_returns: bool, defaults to False
     :return: annualised exponentially-weighted mean (daily) return of each asset
     :rtype: pd.Series
     """
     if not isinstance(prices, pd.DataFrame):
         warnings.warn("prices are not in a dataframe", RuntimeWarning)
         prices = pd.DataFrame(prices)
+
     if returns_data:
         returns = prices
     else:
-        returns = returns_from_prices(prices)
+        returns = returns_from_prices(prices, log_returns)
 
     if compounding:
         return (1 + returns.ewm(span=span).mean().iloc[-1]) ** frequency - 1
@@ -166,12 +178,6 @@ def ema_historical_return(
         return returns.ewm(span=span).mean().iloc[-1] * frequency
 
 
-def james_stein_shrinkage(prices, returns_data=False, compounding=True, frequency=252):
-    raise NotImplementedError(
-        "Deprecated because its implementation here was misguided."
-    )
-
-
 def capm_return(
     prices,
     market_prices=None,
@@ -179,6 +185,7 @@ def capm_return(
     risk_free_rate=0.02,
     compounding=True,
     frequency=252,
+    log_returns=False,
 ):
     """
     Compute a return estimate using the Capital Asset Pricing Model. Under the CAPM,
@@ -207,21 +214,25 @@ def capm_return(
     :param frequency: number of time periods in a year, defaults to 252 (the number
                         of trading days in a year)
     :type frequency: int, optional
+    :param log_returns: whether to compute using log returns
+    :type log_returns: bool, defaults to False
     :return: annualised return estimate
     :rtype: pd.Series
     """
     if not isinstance(prices, pd.DataFrame):
         warnings.warn("prices are not in a dataframe", RuntimeWarning)
         prices = pd.DataFrame(prices)
+
+    market_returns = None
+
     if returns_data:
-        returns = prices
-        market_returns = market_prices
-    else:
-        returns = returns_from_prices(prices)
+        returns = prices.copy()
         if market_prices is not None:
-            market_returns = returns_from_prices(market_prices)
-        else:
-            market_returns = None
+            market_returns = market_prices
+    else:
+        returns = returns_from_prices(prices, log_returns)
+        if market_prices is not None:
+            market_returns = returns_from_prices(market_prices, log_returns)
     # Use the equally-weighted dataset as a proxy for the market
     if market_returns is None:
         # Append market return to right and compute sample covariance matrix
